@@ -204,7 +204,7 @@ nvc0_blend_state_delete(struct pipe_context *pipe, void *hwcso)
     FREE(hwcso);
 }
 
-/* NOTE: ignoring line_last_pixel, using FALSE (set on screen init) */
+/* NOTE: ignoring line_last_pixel */
 static void *
 nvc0_rasterizer_state_create(struct pipe_context *pipe,
                              const struct pipe_rasterizer_state *cso)
@@ -313,6 +313,10 @@ nvc0_rasterizer_state_create(struct pipe_context *pipe,
     SB_BEGIN_3D(so, VIEW_VOLUME_CLIP_CTRL, 1);
     SB_DATA    (so, reg);
 
+    SB_IMMED_3D(so, DEPTH_CLIP_NEGATIVE_Z, cso->clip_halfz);
+
+    SB_IMMED_3D(so, PIXEL_CENTER_INTEGER, !cso->half_pixel_center);
+
     assert(so->size <= (sizeof(so->state) / sizeof(so->state[0])));
     return (void *)so;
 }
@@ -914,10 +918,17 @@ nvc0_set_scissor_states(struct pipe_context *pipe,
                         unsigned num_scissors,
                         const struct pipe_scissor_state *scissor)
 {
-    struct nvc0_context *nvc0 = nvc0_context(pipe);
+   struct nvc0_context *nvc0 = nvc0_context(pipe);
+   int i;
 
-    nvc0->scissor = *scissor;
-    nvc0->dirty |= NVC0_NEW_SCISSOR;
+   assert(start_slot + num_scissors <= NVC0_MAX_VIEWPORTS);
+   for (i = 0; i < num_scissors; i++) {
+      if (!memcmp(&nvc0->scissors[start_slot + i], &scissor[i], sizeof(*scissor)))
+         continue;
+      nvc0->scissors[start_slot + i] = scissor[i];
+      nvc0->scissors_dirty |= 1 << (start_slot + i);
+      nvc0->dirty |= NVC0_NEW_SCISSOR;
+   }
 }
 
 static void
@@ -926,10 +937,18 @@ nvc0_set_viewport_states(struct pipe_context *pipe,
                          unsigned num_viewports,
                          const struct pipe_viewport_state *vpt)
 {
-    struct nvc0_context *nvc0 = nvc0_context(pipe);
+   struct nvc0_context *nvc0 = nvc0_context(pipe);
+   int i;
+
+   assert(start_slot + num_viewports <= NVC0_MAX_VIEWPORTS);
+   for (i = 0; i < num_viewports; i++) {
+      if (!memcmp(&nvc0->viewports[start_slot + i], &vpt[i], sizeof(*vpt)))
+         continue;
+      nvc0->viewports[start_slot + i] = vpt[i];
+      nvc0->viewports_dirty |= 1 << (start_slot + i);
+      nvc0->dirty |= NVC0_NEW_VIEWPORT;
+   }
 
-    nvc0->viewport = *vpt;
-    nvc0->dirty |= NVC0_NEW_VIEWPORT;
 }
 
 static void
@@ -954,7 +973,7 @@ nvc0_set_vertex_buffers(struct pipe_context *pipe,
 
        if (vb[i].user_buffer) {
           nvc0->vbo_user |= 1 << dst_index;
-          if (!vb[i].stride)
+          if (!vb[i].stride && nvc0->screen->eng3d->oclass < GM107_3D_CLASS)
              nvc0->constant_vbos |= 1 << dst_index;
           else
              nvc0->constant_vbos &= ~(1 << dst_index);
@@ -1012,7 +1031,7 @@ nvc0_so_target_create(struct pipe_context *pipe,
    if (!targ)
       return NULL;
 
-   targ->pq = pipe->create_query(pipe, NVC0_QUERY_TFB_BUFFER_OFFSET);
+   targ->pq = pipe->create_query(pipe, NVC0_QUERY_TFB_BUFFER_OFFSET, 0);
    if (!targ->pq) {
       FREE(targ);
       return NULL;
