@@ -31,9 +31,9 @@
 #include <stdio.h>
 
 #include "main/glheader.h"
-#include "program/register_allocate.h"
+#include "util/register_allocate.h"
 #include "util/u_memory.h"
-#include "ralloc.h"
+#include "util/ralloc.h"
 
 #include "r300_fragprog_swizzle.h"
 #include "radeon_compiler.h"
@@ -383,7 +383,12 @@ static enum rc_reg_class variable_get_class(
 							RC_INSTRUCTION_PAIR ) {
 						old_swizzle = r.U.P.Arg->Swizzle;
 					} else {
-						old_swizzle = r.U.I.Src->Swizzle;
+						/* Source operands of TEX
+						 * instructions can't be
+						 * swizzle on r300/r400 GPUs.
+						 */
+						can_change_writemask = 0;
+						break;
 					}
 					new_swizzle = rc_adjust_channels(
 						old_swizzle, conversion_swizzle);
@@ -564,14 +569,16 @@ static void do_advanced_regalloc(struct regalloc_state * s)
 	graph = ra_alloc_interference_graph(ra_state->regs,
 						node_count + s->NumInputs);
 
+	for (node_index = 0; node_index < node_count; node_index++) {
+		ra_set_node_class(graph, node_index, node_classes[node_index]);
+	}
+
 	/* Build the interference graph */
 	for (var_ptr = variables, node_index = 0; var_ptr;
 					var_ptr = var_ptr->Next,node_index++) {
 		struct rc_list * a, * b;
 		unsigned int b_index;
 
-		ra_set_node_class(graph, node_index, node_classes[node_index]);
-
 		for (a = var_ptr, b = var_ptr->Next, b_index = node_index + 1;
 						b; b = b->Next, b_index++) {
 			struct rc_variable * var_a = a->Item;
@@ -609,7 +616,7 @@ static void do_advanced_regalloc(struct regalloc_state * s)
 		input_node++;
 	}
 
-	if (!ra_allocate_no_spills(graph)) {
+	if (!ra_allocate(graph)) {
 		rc_error(s->C, "Ran out of hardware temporaries\n");
 		return;
 	}
