@@ -34,9 +34,11 @@
 #define BRW_EU_H
 
 #include <stdbool.h>
+#include "brw_inst.h"
 #include "brw_structs.h"
 #include "brw_defines.h"
 #include "brw_reg.h"
+#include "intel_asm_annotation.h"
 #include "program/prog_instruction.h"
 
 #ifdef __cplusplus
@@ -45,8 +47,14 @@ extern "C" {
 
 #define BRW_EU_MAX_INSN_STACK 5
 
+/* A helper for accessing the last instruction emitted.  This makes it easy
+ * to set various bits on an instruction without having to create temporary
+ * variable and assign the emitted instruction to those.
+ */
+#define brw_last_inst (&p->store[p->nr_insn - 1])
+
 struct brw_compile {
-   struct brw_instruction *store;
+   brw_inst *store;
    int store_size;
    unsigned nr_insn;
    unsigned int next_insn_offset;
@@ -55,11 +63,10 @@ struct brw_compile {
 
    /* Allow clients to push/pop instruction state:
     */
-   struct brw_instruction stack[BRW_EU_MAX_INSN_STACK];
+   brw_inst stack[BRW_EU_MAX_INSN_STACK];
    bool compressed_stack[BRW_EU_MAX_INSN_STACK];
-   struct brw_instruction *current;
+   brw_inst *current;
 
-   unsigned flag_value;
    bool single_program_flow;
    bool compressed;
    struct brw_context *brw;
@@ -90,34 +97,26 @@ struct brw_compile {
    int loop_stack_array_size;
 };
 
-static inline struct brw_instruction *current_insn( struct brw_compile *p)
-{
-   return &p->store[p->nr_insn];
-}
-
 void brw_pop_insn_state( struct brw_compile *p );
 void brw_push_insn_state( struct brw_compile *p );
-void brw_set_mask_control( struct brw_compile *p, unsigned value );
-void brw_set_saturate( struct brw_compile *p, bool enable );
-void brw_set_access_mode( struct brw_compile *p, unsigned access_mode );
-void brw_set_compression_control(struct brw_compile *p, enum brw_compression c);
-void brw_set_predicate_control_flag_value( struct brw_compile *p, unsigned value );
-void brw_set_predicate_control( struct brw_compile *p, unsigned pc );
-void brw_set_predicate_inverse(struct brw_compile *p, bool predicate_inverse);
-void brw_set_conditionalmod( struct brw_compile *p, unsigned conditional );
-void brw_set_flag_reg(struct brw_compile *p, int reg, int subreg);
-void brw_set_acc_write_control(struct brw_compile *p, unsigned value);
+void brw_set_default_mask_control( struct brw_compile *p, unsigned value );
+void brw_set_default_saturate( struct brw_compile *p, bool enable );
+void brw_set_default_access_mode( struct brw_compile *p, unsigned access_mode );
+void brw_set_default_compression_control(struct brw_compile *p, enum brw_compression c);
+void brw_set_default_predicate_control( struct brw_compile *p, unsigned pc );
+void brw_set_default_predicate_inverse(struct brw_compile *p, bool predicate_inverse);
+void brw_set_default_flag_reg(struct brw_compile *p, int reg, int subreg);
+void brw_set_default_acc_write_control(struct brw_compile *p, unsigned value);
 
 void brw_init_compile(struct brw_context *, struct brw_compile *p,
 		      void *mem_ctx);
-void brw_dump_compile(struct brw_compile *p, FILE *out, int start, int end);
+void brw_disassemble(struct brw_context *brw, void *assembly,
+                     int start, int end, FILE *out);
 const unsigned *brw_get_program( struct brw_compile *p, unsigned *sz );
 
-struct brw_instruction *brw_next_insn(struct brw_compile *p, unsigned opcode);
-void brw_set_dest(struct brw_compile *p, struct brw_instruction *insn,
-		  struct brw_reg dest);
-void brw_set_src0(struct brw_compile *p, struct brw_instruction *insn,
-		  struct brw_reg reg);
+brw_inst *brw_next_insn(struct brw_compile *p, unsigned opcode);
+void brw_set_dest(struct brw_compile *p, brw_inst *insn, struct brw_reg dest);
+void brw_set_src0(struct brw_compile *p, brw_inst *insn, struct brw_reg reg);
 
 void gen6_resolve_implied_move(struct brw_compile *p,
 			       struct brw_reg *src,
@@ -125,22 +124,22 @@ void gen6_resolve_implied_move(struct brw_compile *p,
 
 /* Helpers for regular instructions:
  */
-#define ALU1(OP)					\
-struct brw_instruction *brw_##OP(struct brw_compile *p,	\
-	      struct brw_reg dest,			\
+#define ALU1(OP)				\
+brw_inst *brw_##OP(struct brw_compile *p,	\
+	      struct brw_reg dest,		\
 	      struct brw_reg src0);
 
-#define ALU2(OP)					\
-struct brw_instruction *brw_##OP(struct brw_compile *p,	\
-	      struct brw_reg dest,			\
-	      struct brw_reg src0,			\
+#define ALU2(OP)				\
+brw_inst *brw_##OP(struct brw_compile *p,	\
+	      struct brw_reg dest,		\
+	      struct brw_reg src0,		\
 	      struct brw_reg src1);
 
-#define ALU3(OP)					\
-struct brw_instruction *brw_##OP(struct brw_compile *p,	\
-	      struct brw_reg dest,			\
-	      struct brw_reg src0,			\
-	      struct brw_reg src1,			\
+#define ALU3(OP)				\
+brw_inst *brw_##OP(struct brw_compile *p,	\
+	      struct brw_reg dest,		\
+	      struct brw_reg src0,		\
+	      struct brw_reg src1,		\
 	      struct brw_reg src2);
 
 #define ROUND(OP) \
@@ -157,7 +156,6 @@ ALU2(SHL)
 ALU2(ASR)
 ALU1(F32TO16)
 ALU1(F16TO32)
-ALU2(JMPI)
 ALU2(ADD)
 ALU2(AVG)
 ALU2(MUL)
@@ -197,7 +195,7 @@ ROUND(RNDE)
 /* Helpers for SEND instruction:
  */
 void brw_set_sampler_message(struct brw_compile *p,
-                             struct brw_instruction *insn,
+                             brw_inst *insn,
                              unsigned binding_table_index,
                              unsigned sampler,
                              unsigned msg_type,
@@ -207,8 +205,13 @@ void brw_set_sampler_message(struct brw_compile *p,
                              unsigned simd_mode,
                              unsigned return_format);
 
+void brw_set_indirect_send_descriptor(struct brw_compile *p,
+                                      brw_inst *insn,
+                                      unsigned sfid,
+                                      struct brw_reg descriptor);
+
 void brw_set_dp_read_message(struct brw_compile *p,
-			     struct brw_instruction *insn,
+			     brw_inst *insn,
 			     unsigned binding_table_index,
 			     unsigned msg_control,
 			     unsigned msg_type,
@@ -218,7 +221,7 @@ void brw_set_dp_read_message(struct brw_compile *p,
 			     unsigned response_length);
 
 void brw_set_dp_write_message(struct brw_compile *p,
-			      struct brw_instruction *insn,
+			      brw_inst *insn,
 			      unsigned binding_table_index,
 			      unsigned msg_control,
 			      unsigned msg_type,
@@ -256,8 +259,8 @@ void brw_svb_write(struct brw_compile *p,
 
 void brw_fb_WRITE(struct brw_compile *p,
 		  int dispatch_width,
-		   unsigned msg_reg_nr,
-		   struct brw_reg src0,
+		   struct brw_reg payload,
+		   struct brw_reg implied_header,
 		   unsigned msg_control,
 		   unsigned binding_table_index,
 		   unsigned msg_length,
@@ -278,15 +281,19 @@ void brw_SAMPLE(struct brw_compile *p,
 		unsigned simd_mode,
 		unsigned return_format);
 
-void brw_math( struct brw_compile *p,
+void brw_adjust_sampler_state_pointer(struct brw_compile *p,
+                                      struct brw_reg header,
+                                      struct brw_reg sampler_index,
+                                      struct brw_reg scratch);
+
+void gen4_math(struct brw_compile *p,
 	       struct brw_reg dest,
 	       unsigned function,
 	       unsigned msg_reg_nr,
 	       struct brw_reg src,
-	       unsigned data_type,
 	       unsigned precision );
 
-void brw_math2(struct brw_compile *p,
+void gen6_math(struct brw_compile *p,
 	       struct brw_reg dest,
 	       unsigned function,
 	       struct brw_reg src0,
@@ -318,38 +325,59 @@ void brw_shader_time_add(struct brw_compile *p,
                          struct brw_reg payload,
                          uint32_t surf_index);
 
+/**
+ * Return the generation-specific jump distance scaling factor.
+ *
+ * Given the number of instructions to jump, we need to scale by
+ * some number to obtain the actual jump distance to program in an
+ * instruction.
+ */
+static inline unsigned
+brw_jump_scale(const struct brw_context *brw)
+{
+   /* Broadwell measures jump targets in bytes. */
+   if (brw->gen >= 8)
+      return 16;
+
+   /* Ironlake and later measure jump targets in 64-bit data chunks (in order
+    * (to support compaction), so each 128-bit instruction requires 2 chunks.
+    */
+   if (brw->gen >= 5)
+      return 2;
+
+   /* Gen4 simply uses the number of 128-bit instructions. */
+   return 1;
+}
+
 /* If/else/endif.  Works by manipulating the execution flags on each
  * channel.
  */
-struct brw_instruction *brw_IF(struct brw_compile *p,
-			       unsigned execute_size);
-struct brw_instruction *gen6_IF(struct brw_compile *p, uint32_t conditional,
-				struct brw_reg src0, struct brw_reg src1);
+brw_inst *brw_IF(struct brw_compile *p, unsigned execute_size);
+brw_inst *gen6_IF(struct brw_compile *p, enum brw_conditional_mod conditional,
+                  struct brw_reg src0, struct brw_reg src1);
 
 void brw_ELSE(struct brw_compile *p);
 void brw_ENDIF(struct brw_compile *p);
 
 /* DO/WHILE loops:
  */
-struct brw_instruction *brw_DO(struct brw_compile *p,
-			       unsigned execute_size);
+brw_inst *brw_DO(struct brw_compile *p, unsigned execute_size);
 
-struct brw_instruction *brw_WHILE(struct brw_compile *p);
+brw_inst *brw_WHILE(struct brw_compile *p);
+
+brw_inst *brw_BREAK(struct brw_compile *p);
+brw_inst *brw_CONT(struct brw_compile *p);
+brw_inst *gen6_HALT(struct brw_compile *p);
 
-struct brw_instruction *brw_BREAK(struct brw_compile *p);
-struct brw_instruction *brw_CONT(struct brw_compile *p);
-struct brw_instruction *gen6_CONT(struct brw_compile *p);
-struct brw_instruction *gen6_HALT(struct brw_compile *p);
 /* Forward jumps:
  */
 void brw_land_fwd_jump(struct brw_compile *p, int jmp_insn_idx);
 
-
+brw_inst *brw_JMPI(struct brw_compile *p, struct brw_reg index,
+                   unsigned predicate_control);
 
 void brw_NOP(struct brw_compile *p);
 
-void brw_WAIT(struct brw_compile *p);
-
 /* Special case: there is never a destination, execution size will be
  * taken from src0:
  */
@@ -362,7 +390,7 @@ void brw_CMP(struct brw_compile *p,
 void
 brw_untyped_atomic(struct brw_compile *p,
                    struct brw_reg dest,
-                   struct brw_reg mrf,
+                   struct brw_reg payload,
                    unsigned atomic_op,
                    unsigned bind_table_index,
                    unsigned msg_length,
@@ -376,6 +404,16 @@ brw_untyped_surface_read(struct brw_compile *p,
                          unsigned msg_length,
                          unsigned response_length);
 
+void
+brw_pixel_interpolator_query(struct brw_compile *p,
+                             struct brw_reg dest,
+                             struct brw_reg mrf,
+                             bool noperspective,
+                             unsigned mode,
+                             unsigned data,
+                             unsigned msg_length,
+                             unsigned response_length);
+
 /***********************************************************************
  * brw_eu_util.c:
  */
@@ -404,27 +442,34 @@ void brw_math_invert( struct brw_compile *p,
 		      struct brw_reg dst,
 		      struct brw_reg src);
 
-void brw_set_src1(struct brw_compile *p,
-		  struct brw_instruction *insn,
-		  struct brw_reg reg);
+void brw_set_src1(struct brw_compile *p, brw_inst *insn, struct brw_reg reg);
 
 void brw_set_uip_jip(struct brw_compile *p);
 
-uint32_t brw_swap_cmod(uint32_t cmod);
+enum brw_conditional_mod brw_swap_cmod(uint32_t cmod);
 
 /* brw_eu_compact.c */
 void brw_init_compaction_tables(struct brw_context *brw);
-void brw_compact_instructions(struct brw_compile *p);
-void brw_uncompact_instruction(struct brw_context *brw,
-			       struct brw_instruction *dst,
-			       struct brw_compact_instruction *src);
-bool brw_try_compact_instruction(struct brw_compile *p,
-                                 struct brw_compact_instruction *dst,
-                                 struct brw_instruction *src);
+void brw_compact_instructions(struct brw_compile *p, int start_offset,
+                              int num_annotations, struct annotation *annotation);
+void brw_uncompact_instruction(struct brw_context *brw, brw_inst *dst,
+                               brw_compact_inst *src);
+bool brw_try_compact_instruction(struct brw_context *brw, brw_compact_inst *dst,
+                                 brw_inst *src);
 
-void brw_debug_compact_uncompact(struct brw_context *brw,
-				 struct brw_instruction *orig,
-				 struct brw_instruction *uncompacted);
+void brw_debug_compact_uncompact(struct brw_context *brw, brw_inst *orig,
+                                 brw_inst *uncompacted);
+
+static inline int
+next_offset(const struct brw_context *brw, void *store, int offset)
+{
+   brw_inst *insn = (brw_inst *)((char *)store + offset);
+
+   if (brw_inst_cmpt_control(brw, insn))
+      return offset + 8;
+   else
+      return offset + 16;
+}
 
 #ifdef __cplusplus
 }
